function castSprites()
	-- Sort sprites from far to close
	for i = 1, #sprites, 1 do
		spriteOrder[i] = i
		spriteDistance[i] = ((player.posX - sprites[i].x) * (player.posX - sprites[i].x) + (player.posY - sprites[i].y) * (player.posY - sprites[i].y)); --sqrt not taken, unneeded
	end
	combSort(spriteOrder, spriteDistance, #sprites);
		 
	-- After sorting the sprites, do the projection and draw them
	for i = 1, #sprites, 1 do
		-- Translate sprite position to relative to camera
		local spriteX = sprites[spriteOrder[i]].x - player.posX;
		local spriteY = sprites[spriteOrder[i]].y - player.posY;
				 
		-- Transform sprite with the inverse camera matrix
		local invDet = 1.0 / (player.planeX * player.dirY - player.dirX * player.planeY); -- Required for correct matrix multiplication
		
		local transformX = invDet * (player.dirY * spriteX - player.dirX * spriteY);
		local transformY = invDet * (-player.planeY * spriteX + player.planeX * spriteY); -- This is actually the depth inside the screen, that what Z is in 3D       

		local vMoveScreen = 0

		if sprites[spriteOrder[i]].id == 5 then
			vMoveScreen = curSize / transformY
		end
					
		local spriteScreenX = math.floor((w / 2) * (1 + transformX / transformY));
		
		-- Calculate height of the sprite on screen
		local spriteHeight = math.abs(math.floor(h / (transformY))); -- Using "transformY" instead of the real distance prevents fisheye

		if sprites[spriteOrder[i]].id == 5 then
			spriteHeight = spriteHeight / 2.5
		end

		-- Calculate lowest and highest pixel to fill in current stripe
		local drawStartY = -spriteHeight / 2 + ((h / 2) + TILT) + vMoveScreen;

		local drawEndY = spriteHeight / 2 + ((h / 2) + TILT) + vMoveScreen;

		-- Calculate width of the sprite
		local spriteWidth = math.abs( math.floor (h / (transformY)));

		if sprites[spriteOrder[i]].id == 5 then
			spriteWidth = spriteWidth / 2.5
		end

		local drawStartX = -spriteWidth / 2 + spriteScreenX;

		local drawEndX = spriteWidth / 2 + spriteScreenX;
			
		-- Loop through every vertical stripe of the sprite on screen
		for stripe = math.floor(drawStartX), drawEndX, 1 do
			local texX = math.floor((stripe - (-spriteWidth / 2 + spriteScreenX)) * textureWidth / spriteWidth);

			if texX < 0 then texX = 0 end
				-- The conditions in the if are:
				--1) It's in front of camera plane so you don't see things behind you
				--2) It's on the screen (left)
				--3) It's on the screen (right)
				--4) zBuffer, with perpendicular distance

			if(transformY > 0 and stripe > 0 and stripe < w and transformY < zBuffer[stripe]) then
				local deathColor = 1 - (transformY / maxFogDist)
				if deathColor < 0 then deathColor = 0 end

				if type(screenSprite[stripe]) ~= "table" then
					screenSprite[stripe] = {}
				end

				local temp = {
					x = stripe,
					hor = texX,
					y = drawStartY,
					scale = (drawEndY - drawStartY) / textureHeight,
					id = sprites[spriteOrder[i]].id,
					dc = deathColor,
				}
				if temp.id == 4 then
					temp.scale = (drawEndY - drawStartY) / (textureHeight / 2)
					temp.y = drawStartY - (textureHeight * temp.scale / 2)
				end

				table.insert(screenSprite[stripe], temp)
			end
		end
	end
end