love.graphics.setDefaultFilter("nearest", "nearest")

-- Loads everything
require("cde/load")

-- Misc. functions
require("cde/misc")

-- Floor related functions
require("cde/floor")

-- The majority of raycasting-related functions
require("cde/raycasting")

-- Spritecasting
require("cde/spritecasting")

function love.load()
	loadEverything()
	
	TILT = 0
	tLim = 160
end

function love.update(dt)
	screen = {}
	screenSprite = {}
	furthestWall = 0

	for x = 0, w, 1 do
		castRay(x)
	end

	castSprites()

	if not curShrink then
		curSize = curSize + dt * 50

		if curSize > maxSize then
			curShrink = true
		end
	else
		curSize = curSize - dt * 50

		if curSize < minSize then
			curShrink = false
		end
	end
		
	-- Key Input
	local moveSpeed = dt * player.moveMod
	local rotSpeed = dt * player.rotMod
	local strafeSpeed = dt * player.strafeMod

	if love.keyboard.isDown("w") then
		if (map[math.floor(player.posX + player.dirX * moveSpeed)][math.floor(player.posY)] == 0) then
			player.posX = player.posX + player.dirX * moveSpeed
		end
		if (map[math.floor(player.posX)][math.floor(player.posY + player.dirY * moveSpeed)] == 0) then
			player.posY = player.posY + player.dirY * moveSpeed
		end
	end
	if love.keyboard.isDown("s") then
		if (map[math.floor(player.posX - player.dirX * moveSpeed)][math.floor(player.posY)] == 0) then
			player.posX = player.posX - player.dirX * moveSpeed
		end
		if (map[math.floor(player.posX)][math.floor(player.posY - player.dirY * moveSpeed)] == 0) then
			player.posY = player.posY - player.dirY * moveSpeed
		end
	end
	if love.keyboard.isDown("d") then
		if (map[math.floor(player.posX + player.planeX * moveSpeed)][math.floor(player.posY)] == 0) then
			player.posX = player.posX + player.planeX * strafeSpeed
		end
		if (map[math.floor(player.posX)][math.floor(player.posY + player.planeY * moveSpeed)] == 0) then
			player.posY = player.posY + player.planeY * strafeSpeed
		end
	end
	if love.keyboard.isDown("a") then
		if (map[math.floor(player.posX - player.planeX * moveSpeed)][math.floor(player.posY)] == 0) then
			player.posX = player.posX - player.planeX * strafeSpeed
		end
		if (map[math.floor(player.posX)][math.floor(player.posY - player.planeY * moveSpeed)] == 0) then
			player.posY = player.posY - player.planeY * strafeSpeed
		end 
	end

	if grabbed then
		local mouseX = love.mouse.getX()
		local mouseY = love.mouse.getY()

		if mouseX ~= w/2 then
			local turnSpeed = 0
			if mouseX > w/2 then turnSpeed = rotSpeed * (w/2 - mouseX) end
			if mouseX < w/2 then turnSpeed = -rotSpeed * (mouseX - (w/2)) end

			local oldDirX = player.dirX
			player.dirX = player.dirX * math.cos(turnSpeed) - player.dirY * math.sin(turnSpeed)
			player.dirY = oldDirX * math.sin(turnSpeed) + player.dirY * math.cos(turnSpeed)
			local oldPlaneX = player.planeX
			player.planeX = player.planeX * math.cos(turnSpeed) - player.planeY * math.sin(turnSpeed)
			player.planeY = oldPlaneX * math.sin(turnSpeed) + player.planeY * math.cos(turnSpeed)
		end
		
		--[[ if mouseY ~= h/2 then
			local tiltSpeed = 0
			if mouseY > h/2 then tiltSpeed = 400 * (h/2 - mouseY) end
			if mouseY < h/2 then tiltSpeed = -400 * (mouseY - (h/2)) end

			if TILT + tiltSpeed * dt > -tLim and TILT + tiltSpeed * dt < tLim then
				TILT = TILT + tiltSpeed * dt
			end
		end ]]--

		love.mouse.setPosition(w/2, h/2)
	end
end

function love.draw()
	canvas:clear( )

	processFloor()
	love.graphics.setCanvas(canvas)
	
	drawFloor()

	for x = 0, w, 1 do
		love.graphics.setColor(drawScreenLineColor[x])

		if img[screen[x].texture] then
			love.graphics.draw(img[screen[x].texture], quads[screen[x].hor], screen[x].x, screen[x].start, 0, 1, screen[x].scale)
		end

		if type(topScreen[x]) == "table" then
			for i = #topScreen[x], 1, -1 do
				love.graphics.setColor(topScreen[x][i].color)
				love.graphics.draw(img[topScreen[x][i].texture], quads[topScreen[x][i].hor], topScreen[x][i].x, topScreen[x][i].start, 0, 1, topScreen[x][i].scale)
			end
		end

		if type(screenSprite[x]) == "table" then
			for i = 1, #screenSprite[x] do
				love.graphics.setColor(255 * screenSprite[x][i].dc, 255 * screenSprite[x][i].dc, 255 * screenSprite[x][i].dc)
				love.graphics.draw(spriteImg[screenSprite[x][i].id], quads[screenSprite[x][i].hor], screenSprite[x][i].x, screenSprite[x][i].y, 0, 1, screenSprite[x][i].scale)
			end
		end
	end
	topScreen = {}

	love.graphics.setCanvas()

	love.graphics.setColor(255, 255, 255)
	love.graphics.draw(canvas, 0, 0)


	love.graphics.setColor(255, 20, 20)
	love.graphics.print(string.format("FPS: %1.f\nPlayer Rotation: %1.f\nX: %.1f\nY: %.1f", love.timer.getFPS(), math.deg(getPlayerRot()), player.posX, player.posY))
end

function love.keypressed(key)
	if key == "escape" then love.event.quit() end
	if key == "tab" then grabbed = not grabbed end
end